
;**************************************************************************
;*                    I2C Teplomer                                        *
;**************************************************************************
;**************************************************************************
;*          v 1.16 - 21.02.2015                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1826 used                                       *
;*          OSC.......: Internal osc. used                                *
;**************************************************************************

;--------------------------------------------------------------------------
;power:   3,6 V Li-Ion Accut
;current: 2,0 mA @ 3,6V / max. 3,0 mA during I2C communication with sensor (short time)
;--------------------------------------------------------------------------


 INCLUDE "P16F1826.INC"


_U001			     EQU     B'00100110000100'		;see Page 44 of Datasheet (41391D.pdf)
_U002			     EQU     B'01100011111111'		;see Page 45 of Datasheet (41391D.pdf)


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_d1		EQU	H'22'
v_d2		EQU	H'23'
v_d3		EQU	H'24'

v_cx1		EQU	H'25'
v_cx2		EQU	H'26'
v_short		EQU	H'27'

v_temp1		EQU	H'28'
v_temp2		EQU	H'29'

v_i2c_add	EQU	H'2A'		;I2C address of the temperature sensor

v_tst1		EQU	H'2B'
v_tst2		EQU	H'2C'
v_tst3		EQU	H'2D'
v_tempx		EQU	H'2E'

v_trc		EQU	H'2F'

v_curr_p	EQU	H'30'
v_tof		EQU	H'31'

;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	4
		goto	ir_main		;interrupt service routine


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Interrupt service routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ir_main		clrf	BSR
		bcf	INTCON,GIE

		movlw	D'006'
		movwf	PR2					;overflow after 006 pulses

;-----------------------------------------------------------------------------
		movf	v_curr_p,0
		xorlw	D'000'
		btfsc	STATUS,Z
		call	ishow_digit1		;switch digit 1 on

		movf	v_curr_p,0
		xorlw	D'001'
		btfsc	STATUS,Z
		call	disp_off		;switch digit 1 off (for next 3 cycles)

		movf	v_curr_p,0
		xorlw	D'004'
		btfsc	STATUS,Z
		call	ishow_digit2		;switch digit 2 on

		movf	v_curr_p,0
		xorlw	D'005'
		btfsc	STATUS,Z
		call	disp_off		;switch digit 2 off (for next 3 cycles)

		movf	v_curr_p,0
		xorlw	D'008'
		btfsc	STATUS,Z
		call	ishow_digit3		;switch digit 3 on

		movf	v_curr_p,0
		xorlw	D'009'
		btfsc	STATUS,Z
		call	disp_off		;switch digit 3 off (for next 3 cycles)

;--------------------------------------------
		incf	v_curr_p,1
		movf	v_curr_p,0
		xorlw	D'012'
		btfsc	STATUS,Z
		clrf	v_curr_p		;start new round
;-----------------------------------------------------------------------------
irs_fin		clrf	TMR2
		bcf	PIR1,TMR2IF
		bsf	INTCON,GIE

		retfie
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ishow_digit1	call	disp_off
		movf	v_d1,0
		call	set_digit
		bcf	LATB,D'000'		;decimal point off
		bcf	PORTA,D'004'

		btfsc	v_short,D'000'
		call	i_set_stime

		return
;-----------------------------------------------------------------------------
ishow_digit2	call	disp_off
		movf	v_d2,0
		call	set_digit
		bsf	LATB,D'000'		;decimal point on
		bcf	PORTA,D'003'

		btfsc	v_short,D'000'
		call	i_set_stime

		return
;-----------------------------------------------------------------------------
ishow_digit3	call	disp_off
		movf	v_d3,0
		call	set_digit
		bcf	LATB,D'000'		;decimal point off
		bcf	PORTA,D'002'

		btfsc	v_tof,D'000'
		call	i_take_over

		return
;-----------------------------------------------------------------------------
disp_off	bsf	PORTA,D'004'
		bsf	PORTA,D'003'
		bsf	PORTA,D'002'
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i_take_over	clrf	v_tof

		movf	v_tst2,0
		movwf	v_d1
		movf	v_tst3,0
		movwf	v_d2

		movlw	'-'
		movwf	v_d3

		movf	v_d1,0
		xorlw	'-'
		btfsc	STATUS,Z
		return

		movlw	'0'
		btfsc	v_temp2,D'007'
		movlw	'5'
		movwf	v_d3

		return
;-----------------------------------------------------------------------------
i_set_stime	movlw	D'003'
		movwf	PR2					;overflow after 003 pulses (short time)

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop

		clrf	BSR
;
		clrf	INTCON		;disable interrupts

		banksel	OPTION_REG	;BANK 1
		movlw	B'00111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELB

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'00000000'
		movwf	TRISA

		movlw	B'00010010'
		movwf	TRISB

;-----------------------------------------------------------------------------------------------------

;default frequency is 500 kHz - we keep it

;-----------------------------------------------------------------------------------------------------
;		movf	OSCCON,0
;		andlw	B'11101111'
;		iorlw	B'01101000'
;		movwf	OSCCON		;set 4MHz internal clock (bit 6:3 = 1101) - Datasheet Page 65
;-----------------------------------------------------------------------------------------------------
		movlw	H'04'
		movwf	BSR		;select BANK 4
		clrf	WPUA		;disable weak pull-ups for PORTA
		clrf	WPUB		;disable weak pull-ups for PORTB


		banksel	LATB
		clrf	LATB

		clrf	BSR		;select BANK 0

		clrf	PORTB
		clrf	PORTA
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
		call	i2c_init

		movlw	B'10010100'
		movwf	v_i2c_add

		movlw	D'255'
		movwf	v_trc
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		movlw	'-'
		movwf	v_d1
		movwf	v_d2
		movwf	v_d3

;-----------------------------------------------------------------------------
		clrf	v_curr_p
		clrf	v_tof

		call	timer2_init
;-----------------------------------------------------------------------------

w_release	movlw	'1'
		movwf	v_d1
		movlw	'1'
		movwf	v_d2
		movlw	'6'
		movwf	v_d3

;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
; Main Loop
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------

ddxx		nop
		call	dr4
		call	dr4

; ---- start the sensor ----------------
		call	tmp75_start
		call	dr2

; --- read the temperature etc. -------
		clrf	v_tof
		call	tmp75_read
		call	temp_corr
		call	t_convert2dec
		bsf	v_tof,D'000'
; -------------------------------------
		goto	ddxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------
;read routine for temperature sensor TCN75 (and TMP100, LM76, AT30TSE754, ...)
;w/o pointer setting
;---------------------------------------------------------------------------------
tmp75_read	nop

;1. read the temperature
		call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'00'
		call	i2c_send		;set pointer to 00h - temperature register

		call	i2c_start

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send		;addressing the device

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

;-----------------------------------------------------
;enter shut-down mode - accu power saving...

		call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'01'
		call	i2c_send		;set pointer to 00h - configuration register

		movlw	B'00000001'		;bit 0 = 1 --> shut-down mode
		call	i2c_send

		call	i2c_stop


		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;enter normal operation
;
tmp75_start	call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'01'
		call	i2c_send		;set pointer to 00h - configuration register

		movlw	B'00000000'		;bit 0 = 0 --> normal operation
		call	i2c_send

		call	i2c_stop

		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
set_digit	movwf	v_cx1
		bcf	v_short,D'000'
;----------------------------------------------------
		movf	v_cx1,0
		xorlw	'-'
		btfsc	STATUS,Z
		goto	wm_minus

		movf	v_cx1,0
		xorlw	' '
		btfsc	STATUS,Z
		goto	wm_space
;----------------------------------------------------
		movf	v_cx1,0
		xorlw	'0'
		btfsc	STATUS,Z
		goto	wm_0

		movf	v_cx1,0
		xorlw	'1'
		btfsc	STATUS,Z
		goto	wm_1

		movf	v_cx1,0
		xorlw	'2'
		btfsc	STATUS,Z
		goto	wm_2

		movf	v_cx1,0
		xorlw	'3'
		btfsc	STATUS,Z
		goto	wm_3

		movf	v_cx1,0
		xorlw	'4'
		btfsc	STATUS,Z
		goto	wm_4

		movf	v_cx1,0
		xorlw	'5'
		btfsc	STATUS,Z
		goto	wm_5

		movf	v_cx1,0
		xorlw	'6'
		btfsc	STATUS,Z
		goto	wm_6

		movf	v_cx1,0
		xorlw	'7'
		btfsc	STATUS,Z
		goto	wm_7

		movf	v_cx1,0
		xorlw	'8'
		btfsc	STATUS,Z
		goto	wm_8

		movf	v_cx1,0
		xorlw	'9'
		btfsc	STATUS,Z
		goto	wm_9
;----------------------------------------------------
		movf	v_cx1,0
		xorlw	D'000'
		btfsc	STATUS,Z
		goto	wm_0

		movf	v_cx1,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	wm_1

		movf	v_cx1,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	wm_2

		movf	v_cx1,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	wm_3

		movf	v_cx1,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	wm_4

		movf	v_cx1,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	wm_5

		movf	v_cx1,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	wm_6

		movf	v_cx1,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	wm_7

		movf	v_cx1,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	wm_8

		movf	v_cx1,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	wm_9
;----------------------------------------------------

		goto	wm_minus
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;------------------------------------------------
wm_space	movf	LATA,0
		andlw	B'00111100'
		iorlw	B'00000000'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'00000000'
		movwf	LATB

		bsf	v_short,D'000'

		return
;------------------------------------------------
wm_minus	movf	LATA,0
		andlw	B'00111100'
		iorlw	B'00000000'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'00100000'
		movwf	LATB

		bsf	v_short,D'000'

		return
;------------------------------------------------
wm_0		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'11000011'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'11000000'
		movwf	LATB
		return
;------------------------------------------------
wm_1		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'10000001'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'00000000'
		movwf	LATB

		bsf	v_short,D'000'

		return
;------------------------------------------------
wm_2		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'01000011'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'10100000'
		movwf	LATB
		return
;------------------------------------------------
wm_3		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'11000011'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'00100000'
		movwf	LATB
		return
;------------------------------------------------
wm_4		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'10000001'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'01100000'
		movwf	LATB
		return
;------------------------------------------------
wm_5		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'11000010'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'01100000'
		movwf	LATB
		return
;------------------------------------------------
wm_6		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'11000010'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'11100000'
		movwf	LATB
		return
;------------------------------------------------
wm_7		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'10000011'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'01000000'
		movwf	LATB
		return
;------------------------------------------------
wm_8		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'11000011'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'11100000'
		movwf	LATB
		return
;------------------------------------------------
wm_9		movf	LATA,0
		andlw	B'00111100'
		iorlw	B'11000011'
		movwf	LATA
		movf	LATB,0
		andlw	B'00011110'
		iorlw	B'01100000'
		movwf	LATB
		return
;------------------------------------------------



;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'032'			;for 500 kHz the value D'032' shall be used (instead of D'255' for 4 MHz)
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c_wait	movlw	D'002'
		goto	dxc

d2c_2		movlw	D'005'
		goto	dxc
d2c		MOVLW   D'002'
		goto	dxc
d1c		MOVLW   D'001'
dxc		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'200'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;I2C
;set up I2C BUS for communication as a master
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
i2c_init	movlw	H'04'
		movwf	BSR		;select BANK 4

		movlw	B'00101000'		;see Datasheet Page 277
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR		;select BANK 0

		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
i2c_start	bsf	BSR,D'002'		;select register BANK 4
		bsf	SSP1CON2, SEN
		bcf	BSR,D'002'		;select register BANK 0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	BSR,D'002'
		bsf	SSP1CON2, PEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	BSR,D'002'
		bcf	SSP1CON2, ACKDT		;ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	BSR,D'002'
		bsf	SSP1CON2, ACKDT		;NOT ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	BSR,D'002'
		bsf	SSP1CON2, RCEN
		bcf	BSR,D'002'
		call	d55
		bsf	BSR,D'002'
		movf	SSP1BUF,0
		bcf	BSR,D'002'
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bsf	BSR,D'002'
		movwf	SSP1BUF
		bcf	BSR,D'002'
		call	d55			;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	BSR,D'002'
		bsf	SSP1CON2, RSEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;----------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------

t_convert2dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3
;
		clrf	v_tempx
;
		movlw	D'255'
		xorwf	v_temp2,0		;if LSB is eq to 255 --- probably no sensor is connected
		btfsc	STATUS,Z
		goto	temp_no_value
;
		btfsc	v_temp1,D'007'		;negative value?
		goto	negative_temp
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;positive value conversion
;
		incf	v_temp1,0		;we convert only MSB now
		movwf	TIMER1			;store "to be converted" value to data location TIMER1
		
t_next		decfsz	TIMER1,1
		goto	t_inc_t3
		goto	finito7

t_inc_t3	call	t_inc_t
		goto	t_next
;we are done.....----
finito7		movf	v_tst1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	finito7_2
;
		movlw	' '		;use space instead of leading '0'
		movwf	v_tst1
;dblchk 2nd position as well
		movf	v_tst2,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	finito7_2
		movlw	' '		;use space instead of leading '0'
		movwf	v_tst2

finito7_2	movlw	'-'
		btfsc	v_temp1,D'007'
		movwf	v_tst1		;minus sign
		return
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;negative value conversion
;
negative_temp	movf	v_temp1,0
		movwf	TIMER1

nt_next		incf	TIMER1,1
		btfsc	STATUS,Z
		goto	finito9
		call	t_inc_t
		goto	nt_next
;------------------------------------
finito9		call	t_inc_t
		goto	finito7
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
t_inc_t		nop

		incf	v_tempx,1

		incf	v_tst3,1
;----------------------------------------
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	t_c_fin
;
		movlw	'0'
		movwf	v_tst3
;----------------------------------------
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	t_c_fin
;
		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

		return
;-----------------------------------
t_c_fin		nop		
		return

;-----------------------------------
temp_no_value	movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

		return
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;harcoded temperature correction
;-----------------------------------------------------------------------------------------
temp_corr	decf	v_temp1,1
		decf	v_temp1,1
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
timer2_init	nop

;TIMER2 setup
		movlw	B'00100001'
		movwf	T2CON

			;prescaler  =  4 / <1:0> = 01		;for 4 MHz clock: TMR is inceremeted 62.500 x pro sec. (1MHz:16 = 62.500)
			;postscaler =  5 / <6:3> = 0100		;after "overflow" every 5 times is interrupt raised
			;timer if off    /  <2> = 0

		clrf	TMR2

		movlw	D'006'
		movwf	PR2					;overflow after 006 pulses

;with this settings interrupt should happen 125x per second


;Interrupt setup
		banksel	PIE1		

		clrf	PIE1
		clrf	PIE2

		bsf	PIE1,TMR2IE		;Enables the TMR2 to PR2 match interrupt

		clrf	BSR		;select BANK 0

		movlw	B'11000000'
		movwf	INTCON

;at this moment is everything prepared - but TIMER2 is not running

		bsf	T2CON,D'002'		;start TIMER2

		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------




;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END